//
//  CameraNativeView.swift
//  Pods
//
//  Created by Sarbjyot Chahal on 31.03.22.
//

import Foundation
import Flutter
#if canImport(ScanbotSDK)
import ScanbotSDK
#else
import ScanbotBarcodeScannerSDK
#endif

class CameraNativeView: NSObject, FlutterPlatformView, CallErrorHandler {
    
    private var containerView: UIView
    private let configuration: [String: Any?]?
    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?
    private var scanner: ScannerDelegate?
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger messenger: FlutterBinaryMessenger
    ) {
        
        detectorChannel = FlutterMethodChannel(name: "scanbot_sdk_camera.detector_\(viewId)",
                                               binaryMessenger: messenger)
        cameraMethodChannel = FlutterMethodChannel(name: "scanbot_sdk_camera_\(viewId)",
                                                   binaryMessenger: messenger)
        finderChannel = FlutterMethodChannel(name: MethodChannelConstants.kClassicalCameraFinderChannel,
                                             binaryMessenger: messenger)
        
        containerView = UIView()
        configuration = args as? [String: Any?]
        
        #if canImport(ScanbotSDK)
        switch configuration!["detectorType"] as? String {
        case "barcode":
            scanner = BarcodeScanner(cameraMethodChannel: cameraMethodChannel,
                                     detectorChannel: detectorChannel,
                                     finderChannel: finderChannel,
                                     configuration: configuration!)
        case "document":
            scanner = DocumentScanner(cameraMethodChannel: cameraMethodChannel,
                                      detectorChannel: detectorChannel,
                                      finderChannel: finderChannel,
                                      configuration: configuration!)
        case "mc":
            scanner = MedicalCertificateScanner(cameraMethodChannel: cameraMethodChannel,
                                                detectorChannel: detectorChannel,
                                                finderChannel: finderChannel,
                                                configuration: configuration!)
        default: break
        }
        #else
        scanner = BarcodeScanner(cameraMethodChannel: cameraMethodChannel,
                                 detectorChannel: detectorChannel,
                                 finderChannel: finderChannel,
                                 configuration: configuration!)
        #endif
        
        super.init()
        self.scanner?.attach(view: containerView)
        
        self.cameraMethodChannel?.setMethodCallHandler({ (call, result) in
            self.handleCameraCall(call: call, result: result)
        })
        
    }
    
    func view() -> UIView {
        return containerView
    }
    
    private func toggleFlash(arguments: [String: Any?]?) {
        if let flashEnabled = arguments?["enabled"] as? Bool {
            self.scanner?.setFlash(enabled: flashEnabled)
        }
    }
    
    private func isFlashAvailable(result: @escaping FlutterResult) {
        result(self.scanner?.isFlashAvailable())
    }
    
    func handleCameraCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as? [String: Any?]
        switch call.method {
        case "flashEnabled": 
            toggleFlash(arguments: arguments)
        case "isFlashAvailable": 
            isFlashAvailable(result: result)
            return
        case "stopPreview": 
            self.scanner?.stopPreview()
        case "resumePreview": 
            self.scanner?.resumePreview()
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        returnSuccess(result: result)
    }
    
    func returnSuccess(result: @escaping FlutterResult) {
        result("success")
    }
}
