//
//  ErrorHandler.swift
//  barcode_scanner
//
//  Created by Seifeddine Bouzid on 15.05.23.
//

import Foundation
import ScanbotBarcodeScannerSDK

protocol FlutterErrorHandler {
    func flutterError(code: Int, message: String, details: ErrorTypes?) -> FlutterError
}

extension FlutterErrorHandler {
    func flutterError(code: Int, message: String, details: ErrorTypes?) -> FlutterError {
        return FlutterError(code: "\(code)",
                            message: "ERROR_IOS: \(message)",
                            details: details != nil ? details!.value : nil)
    }
}

enum ErrorTypes {
    
    case any(_: Any?)
    
    var value: Any? {
        switch self {
        case .any(let detail):
            return detail
        }
    }
}

enum CallError: FlutterErrorHandler {
    
    case argumentsError(method: String, details: Any?)
    case unknownCallError(method: String)
    case customError(message: String, details: Any?)
    
    var value: FlutterError {
        switch self {
        case .argumentsError(let method, let details):
            return self.flutterError(code: 1001,
                                     message: "\(method) was called with wrong arguments",
                                     details: ErrorTypes.any(details))
        case .unknownCallError(let method):
            return self.flutterError(code: 1002,
                                     message: "\(method) is unknown or can't be handled",
                                     details: nil)
        case .customError(let message, let details):
            return self.flutterError(code: 1003, message: message, details: ErrorTypes.any(details))
        }
    }
}
